""" Python File to Invoke ChatBot"""

from __future__ import annotations

import os
from typing import Dict, Optional

from .src.bot import ChatGPT
from .src.utils import EXIT_INPUTS, HELLO_INPUTS, WELCOME_MSG


def get_tokens() -> Dict[str, str]:
    """Fetches Token from Environment Vars"""
    if not (token := os.environ.get("APIKEY_CHATGPT")) or not (
        org_token := os.environ.get("ORG_APIKEY_CHATGPT")
    ):
        raise Exception("TOKEN NOT FOUND at PATH ~/.secrets")
    return dict(token=token, org_token=org_token)


def get_bot(tokens: Optional[Dict[str, str]] = None) -> ChatGPT:
    """
    Create ChatGPT Instance
    Args:
        tokens (Dict[str, str], optional): Token reuqired by ChatGPT. Defaults to get_tokens().
    Returns:
        ChatGPT: ChatGPT Instance
    """
    if not tokens:
        tokens = get_tokens()
    return ChatGPT(token=tokens["token"], org_token=tokens["org_token"])


def chatbot(bot: Optional[ChatGPT] = None) -> None:
    """Invokes Bot and Interact with Users"""
    if not bot:
        bot = get_bot()
    idx: int = 0
    while True:
        idx += 1
        if idx == 1:
            print(WELCOME_MSG)
        print("_" * 100 + "\n")

        user_input: str = input("You:\n>> ")
        if user_input.strip().lower() in EXIT_INPUTS:
            print("\nChatbot:\n>> Goodbye! 👋\n")
            break

        if user_input.strip().lower() in HELLO_INPUTS:
            print("\nChatbot:\n>> Hello, Please ask me anything!\n")
            continue

        response: str = bot.generate_response(user_input)
        print(f"\nChatbot:\n>> {response}\n")


if __name__ == "__main__":
    chatbot()



""" Python Bot using ChatGPT's API"""

from __future__ import annotations

import openai
from openai import Completion
from openai.openai_object import OpenAIObject


class ChatGPT:
    """Wrapper for ChatGPT"""

    def __init__(self, token: str, org_token: str = "", **kwargs) -> None:
        """Initiaze Args
        Args:
            token (str): Token to interact with OpenAI
            organization (str): Organization Token
        """
        openai.api_key = token
        if org_token:
            openai.organization = org_token
        self.engine = kwargs.get("engine", "text-ada-001")
        self.max_token = kwargs.get("max_tokens", 150)

    def generate_response(self, prompt: str) -> str:
        """
        Generate Response
        Args:
            prompt (str): User Prompts
        Returns:
            str: ChatGPT Response
        """
        message: str = "Request cannot be processed"
        completions: OpenAIObject = Completion.create(
            engine=self.engine,
            prompt=prompt,
            max_tokens=self.max_token,
            n=1,
            stop=None,
            temperature=0.5,
        )
        if choices := completions.choices:
            message = choices[0].text.strip()
        return message



"""Utils for ChatBot"""
from __future__ import annotations

from typing import List

EXIT_INPUTS: List[str] = ["bye", "sayonara", "exit", "shutdown", "stop", "close"]
HELLO_INPUTS: List[str] = [
    "hello",
    "hi",
    "hello world",
    "hello chatbot",
    "hello bot",
    "hello",
    "hey",
]
WELCOME_MSG: str = (
    "\n🙏 Welcome to Unoffical ChatGpt Bot 🤖.\n🕵️  I am here to help humanity.\nℹ️  Please ask me anything.\n"
    + "\n🙏 Since I am new, I am learning as much as possible. So, Please co-operate with me."
    + "\n😎 I can Auto complete text, Q&A, Grammer correction, Write code, Language translations, NLP and many more stuffs. Try me !!!\n"
)



"""Package Release"""

from __future__ import annotations

from setuptools import find_packages, setup

setup(
    name="pybot-chatgpt",
    version="2.0.0",
    description="PythonBot wrapping ChatGPT API",
    long_description=open("README.md", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    author="Shubham Raj",
    author_email="shubhamraj2202@gmail.com",
    url="https://github.com/shubhamraj2202/pybot-chatgpt",
    packages=find_packages(include=["chatgpt", "chatgpt.*", "chatgpt.src"]),
    install_requires=["openai"],
    test_requires=["pytest", "mock"],
    project_urls={
        "Bug Reports": "https://github.com/shubhamraj2202/pybot-chatgpt/issues",
    },
    entry_points={
        "console_scripts": [
            "pybot = chatgpt.run:chatbot",
        ],
    },
)
